#!/usr/bin/env python
"""A utility for editing Emu8086 interrupt vector.

Author: Kamran Riaz Khan <krkhan@inspirated.com"""


import gtk
import os
import struct

class IntVectTable:
	INTC = 0
	SEGMENTC = 1
	OFFSETC = 2
	def __init__(self):
		self.table = []
		self.parse_vect_table(None)

		self.liststore = gtk.ListStore(str, str, str)
		self.populate_list_store()

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_title("8086 Interrupt Vector Editor")
		self.window.set_size_request(200, 500)
		self.window.set_border_width(5)
		self.window.connect("delete_event", self.delete_event_cb)

		self.vbox = gtk.VBox(False, 5)
		self.window.add(self.vbox)

		self.openbutton = gtk.Button(stock = gtk.STOCK_OPEN)
		self.openbutton.connect("clicked", self.open_clicked_cb)
		self.vbox.pack_start(self.openbutton, False, False)

		self.scrolledwindow = gtk.ScrolledWindow()
		self.vbox.pack_start(self.scrolledwindow, True, True)

		self.treeview = gtk.TreeView(self.liststore)

		self.tvcint = gtk.TreeViewColumn("INT")
		self.tvcsegment = gtk.TreeViewColumn("Segment")
		self.tvcoffset = gtk.TreeViewColumn("Offset")

		self.treeview.append_column(self.tvcint)
		self.treeview.append_column(self.tvcsegment)
		self.treeview.append_column(self.tvcoffset)

		self.cellint = gtk.CellRendererText()
		self.cellsegment = gtk.CellRendererText()
		self.celloffset = gtk.CellRendererText()

		self.cellsegment.set_property("editable", True)
		self.celloffset.set_property("editable", True)

		self.cellsegment.connect("edited", self.edited_cb, self.SEGMENTC)
		self.celloffset.connect("edited", self.edited_cb, self.OFFSETC)

		self.tvcint.pack_start(self.cellint, True)
		self.tvcint.add_attribute(self.cellint, 'text', self.INTC)

		self.tvcsegment.pack_start(self.cellsegment, True)
		self.tvcsegment.add_attribute(self.cellsegment, 'text', self.SEGMENTC)

		self.tvcoffset.pack_start(self.celloffset, True)
		self.tvcoffset.add_attribute(self.celloffset, 'text', self.OFFSETC)

		self.tvcint.set_sort_column_id(0)

		self.scrolledwindow.add(self.treeview)

		self.savebutton = gtk.Button(stock = gtk.STOCK_SAVE)
		self.vbox.pack_start(self.savebutton, False, False)
		self.savebutton.connect("clicked", self.save_clicked_cb)

		self.savebutton.show()
		self.treeview.show()
		self.scrolledwindow.show()
		self.openbutton.show()
		self.vbox.show()
		self.window.show()

	def parse_vect_table(self, filename = None):
		self.table = []
		if(filename):
			with open(filename) as f:
				data = f.read()
				size = len(data)
				if size < 1024:
					vect_str = data + ("".join(['\x00'] * 1024 - size))
				else:
					vect_str = data[:1024]
		else:
			vect_str = "".join(['\x00'] * 1024)

		for i in range(256):
			row = i * 4
			offset, segment = struct.unpack('<HH', vect_str[row : row + 4])
			self.table.append({'offset' : offset, 'segment' : segment})

	def write_vect_table(self, filename):
		with open(filename, "w") as f:
			data = "".join(
				map(lambda row: struct.pack('<HH', row['offset'], row['segment']),
				self.table))
			f.write(data)
			f.close()

	def populate_list_store(self):
		self.liststore.clear()
		for (interrupt, row) in enumerate(self.table):
			self.liststore.append(("%02X" % interrupt,
				"%04X" % row['segment'],
				"%04X" % row['offset']))

	def run(self):
		gtk.main()

	def delete_event_cb(self, widget, event, data = None):
		gtk.main_quit()
		return True

	def open_clicked_cb(self, widget, data = None):
		dialog = gtk.FileChooserDialog("Open...",
			None,
			gtk.FILE_CHOOSER_ACTION_OPEN,
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
			gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)
		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.parse_vect_table(dialog.get_filename())
			self.populate_list_store()

		dialog.destroy()
		return True

	def save_clicked_cb(self, widget, data = None):
		dialog = gtk.FileChooserDialog("Save...",
			None,
			gtk.FILE_CHOOSER_ACTION_SAVE,
			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
			gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)
		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		dialog.add_filter(filter)
		dialog.set_do_overwrite_confirmation(True)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			filename = dialog.get_filename()
			self.write_vect_table(filename)

		dialog.destroy()
		return True

	def edited_cb(self, cell, path, newtext, col):
		try:
			newvalue = int(newtext, 16)
			if newvalue > 65535:
				raise ValueError
		except ValueError:
			return False
		else:
			validtext = "%04X" % newvalue

		self.liststore[path][col] = validtext

		i, s, o = self.liststore[path]
		interrupt = int(i, 16)
		segment = int(s, 16)
		offset = int(o, 16)
		self.table[interrupt]['segment'] = segment
		self.table[interrupt]['offset'] = offset

		return True

if __name__ == "__main__":
	ivt = IntVectTable()
	ivt.run()

